/** \file Settings.h
*/

#ifndef SettingsH
#define SettingsH
//---------------------------------------------------------------------------
#include <System.hpp>
#include <vector>

/** \brief Common (not related to any dll) program settings with Read/Write functions
*/
class Settings
{
public:
	bool Read(AnsiString asFileName);  ///< write settings to ini file
	bool Write(AnsiString asFileName); ///< read settings from ini file
	int iPosX, iPosY;                  ///< main window coordinates
	int iHeight, iWidth;			   ///< main window size
//	bool bAutoScroll;				    ///< auto scroll log
	unsigned int iMaxUiLogLines;	   ///< maximum no. of lines cached with GUI
	bool bLogToFile;					///< write log to file
	bool bAlwaysOnTop;                 ///< window always on top setting
	bool bWindowMaximized;             ///< was window maximized when app was closing?
	unsigned int iDisplayRefreshFreq;	///< display refresh frequency (if applicable)
	AnsiString asInputDeviceDllName;   ///< name of previously selected device interface dll
	AnsiString asPlotPictureExportDir;	///< default directory to save exported plot picture bitmap
	AnsiString asPlotDataExportDir;		///< default directory to save exported plot as JSON file
    AnsiString asLanguage;

	struct S_GRAPH
	{
		bool bTraceAntialiasing;
		bool bSmoothTraceMoving;
		int ToolboxTransparency;		
	} Graph;
	struct S_GRAPH_VISIBILITY
	{
		bool bGridX;
		bool bGridY;
		bool bGridLabelsX;
		bool bGridLabelsY;
		bool bCursor;
		bool bGndMarker;
		bool bTriggerPosMarker;
		bool bTriggerValMarker;
	} GraphVisibility;
	struct S_GRAPH_COLORS
	{
		std::vector<unsigned int> traces;
    } GraphColors;

	// "Measurements" module settings
	struct S_MEASUREMENTS
	{
		bool bVisible;
		int iPosX, iPosY;
	} Measurements;

	struct S_MS3RECORDING
	{
		int iSizeLimit;		///< recording file size limit [kB]
	} Ms3Recording;

	struct S_FFT
	{
        enum { SAMPLES_HARD_LIMIT = 131072 };
		unsigned int iSamplesLimit;	///< limit of samples used to calculate FFT (will be rounded up to next power of 2)
		AnsiString asWindowName;
	} FFT;

	float fRecorderInterval;
public:
	static const float fRecorderMinInterval;
	static const float fRecorderMaxInterval;
};

extern Settings appSettings;

#endif
