//---------------------------------------------------------------------------

#pragma hdrstop

#include "TestScope.h"
#include "..\miniscope\device\MiniscopeDevice.h"
#include "Signal.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "Mutex.h"
#include "ScopedLock.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

TestScope testScope;
Mutex mutex;
Signal signal[CHANNELS];


extern unsigned int iBuf[];
extern float fSens[];
extern float fSampl[];


TestScope::TestScope():
	workerThread(NULL),
	bManualTrigger(false)
{
	bDeviceOpened = false;
	bRunning = false;
	bStopRequest = false;
	mode = TRIGGER_MODE_AUTO;
	callbackData = NULL;
	callbackLog = NULL;
	callbackConnect = NULL;
	callbackTrigger = NULL;
	settings.samples = 512;
	for (int i=0; i<CHANNELS; i++)
	{
		settings.fDivY[i] = 0.001;
	}
	settings.fDivX = 0.00001;


}

TestScope::~TestScope()
{
}

void TestScope::log(char *lpData, ...)
{
    static char buf[256]; //determines max message length
    va_list ap;
    va_start(ap, lpData);
    vsnprintf(buf, sizeof(buf), lpData, ap);
    va_end(ap);
    buf[sizeof(buf)-1] = 0;

    if (callbackLog)
        callbackLog(buf);
}

int TestScope::Connect(void)
{
	ConnectCb(DEVICE_CONNECTED, "Test signal plugin");
	workerThread = new WorkerThread(false);
	return 0;
}

int TestScope::Disconnect(void)
{
	delete workerThread;
	workerThread = NULL;
	return 0;
}

int TestScope::Run(bool state)
{
	bRunning = state;
	return 0;
}

int TestScope::SetTriggerMode(enum E_TRIGGER_MODE m)
{
	mode = m;
	return 0;
}

int TestScope::ManualTrigger(void)
{
	bManualTrigger = true;
	return 0;
}

int TestScope::SetTriggerType(int channel, enum E_TRIGGER_TYPE type)
{
	return 0;
}

int TestScope::EnableChannel(int channel, bool state)
{
	ScopedLock<Mutex> lock(mutex);
	settings.channelEnabled[channel] = state;
	return 0;
}

int TestScope::SetCouplingType(int channel, enum E_COUPLING_TYPE type)
{
	return 0;
}

int TestScope::SetBufferSize(int size)
{
	if (size >= 14336)
	{
		settings.samples = 14336;
	}
	else if (size >= 8192)
	{
		settings.samples = 8192;
	}
	else if (size >= 4096)
	{
		settings.samples = 4096;
	}
	else if (size >= 2048)
	{
		settings.samples = 2048;
	}
	else if (size >= 1024)
	{
		settings.samples = 1024;
	}
	else
	{
		settings.samples = 512;
	}
	return 0;
}

int TestScope::SetSensitivity(int channel, int id)
{
	if (id < 0 || id > 5)
		return -1;
	settings.fDivY[channel] = fSens[id];
	return 0;
}

int TestScope::SetSampling(int id)
{
	if (id < 0 || id > 10)
		return -1;
	settings.fDivX = fSampl[id];
	return 0;
}

int TestScope::SetTriggerLevel(int level)
{
	if (level < -100 || level > 100)
		return -1;
	settings.triggerval = 255 - ((255 * (level+100))/200);
	return 0;
}

int TestScope::SetTriggerPosition(int pos)
{
	if (pos < 0 || pos > 100)
		return -1;
	settings.triggerpos = pos;
	return 0;
}

void TestScope::handle(void)
{
	if ((bRunning && mode == TRIGGER_MODE_CONTINUOUS) || bManualTrigger)
	{
        ScopedLock<Mutex> lock(mutex);
		callbackTrigger();
		int enabledChannelsCnt = 0;
		for (int i=0; i<CHANNELS; i++)
		{
			if (settings.channelEnabled[i] == false)
				continue;
			short *tmp = signal[i].Generate(settings.samples,
				settings.fDivX, settings.fDivY[i]);
			memcpy(buffer + (enabledChannelsCnt * settings.samples), tmp, sizeof(buffer[0]) * settings.samples);
			enabledChannelsCnt++;
		}
		int size = enabledChannelsCnt * settings.samples * sizeof(short);
		callbackData(buffer, size);
	}
	bManualTrigger = false;	
}

